@@ -251,7 +251,9 @@ def copy(
   if isinstance(x, FrozenDict):
     return x.copy(add_or_replace)
   elif isinstance(x, dict):
-    new_dict = jax.tree_map(lambda x: x, x)  # make a deep copy of dict x
+    new_dict = jax.tree_util.tree_map(
+        lambda x: x, x
+    )  # make a deep copy of dict x
     new_dict.update(add_or_replace)
     return new_dict
   raise TypeError(f'Expected FrozenDict or dict, got {type(x)}')
@@ -280,7 +282,9 @@ def pop(
   if isinstance(x, FrozenDict):
     return x.pop(key)
   elif isinstance(x, dict):
-    new_dict = jax.tree_map(lambda x: x, x)  # make a deep copy of dict x
+    new_dict = jax.tree_util.tree_map(
+        lambda x: x, x
+    )  # make a deep copy of dict x
     value = new_dict.pop(key)
     return new_dict, value
   raise TypeError(f'Expected FrozenDict or dict, got {type(x)}')
@@ -322,7 +326,7 @@ def _frozen_dict_state_dict(xs):
 
 
 def _restore_frozen_dict(xs, states):
-  diff = set(map(str, xs.keys())).difference(states.keys())
+  diff = set(map(str, xs.keys())).difference(map(str, states.keys()))
   if diff:
     raise ValueError(
       'The target dict keys and state dict keys do not match, target dict'
