@@ -790,6 +790,18 @@ def vmap(
   rng_axes = tuple(0 if rng_split else None for rng_split in rng_splits)
 
   def inner(scope_fn, repack_fn, variable_groups, rng_groups, *args):
+    # optional user-defined variable transform on the way in
+    new_variable_groups = []
+    for var_group, axis in zip(variable_groups, variable_in_axes):
+      if axis is not None:
+        new_variable_groups.append(
+            meta.remove_axis(var_group, axis, metadata_params)
+        )
+      else:
+        new_variable_groups.append(var_group)
+    variable_groups = tuple(new_variable_groups)
+
+    # split rngs
     def find_axis_size(axis, x):
       if axis is not None:
         leaves = jax.tree_util.tree_leaves(x)
@@ -797,9 +809,9 @@ def vmap(
           return leaves[0].shape[axis]
       return ()
 
-    # split rngs
     axis_sizes = jax.tree_util.tree_map(
-      find_axis_size, (variable_in_axes, in_axes), (variable_groups, args)
+        find_axis_size, (variable_in_axes, in_axes), (variable_groups, args),
+        is_leaf=lambda x: x is None
     )
     axis_sizes = set(jax.tree_util.tree_leaves(axis_sizes))
     if axis_size is None and len(axis_sizes) == 1:
@@ -810,30 +822,25 @@ def vmap(
       raise ValueError('axis_size should be specified manually.')
     else:
       d_axis_size = axis_size
-    split_fn = lambda rng: random.split(rng, d_axis_size)
+    # random.clone is only available on Jax versions 0.4.26 or newer
+    # see: https://jax.readthedocs.io/en/latest/jax.experimental.key_reuse.html
+    if hasattr(random, 'clone'):
+      split_fn = lambda rng: random.split(random.clone(rng), d_axis_size)
+    else:
+      split_fn = lambda rng: random.split(rng, d_axis_size)
 
     rng_groups = tuple(
-      tree_map_rngs(split_fn, rng_group) if split else rng_group
-      for rng_group, split in zip(rng_groups, rng_splits)
+        tree_map_rngs(split_fn, rng_group) if split else rng_group
+        for rng_group, split in zip(rng_groups, rng_splits)
     )
 
-    new_variable_groups = []
-    for var_group, axis in zip(variable_groups, variable_in_axes):
-      if axis is not None:
-        new_variable_groups.append(
-          meta.remove_axis(var_group, axis, metadata_params)
-        )
-      else:
-        new_variable_groups.append(var_group)
-    variable_groups = tuple(new_variable_groups)
-
     @functools.partial(
-      jax.vmap,
-      in_axes=(variable_in_axes, rng_axes, in_axes),
-      out_axes=(out_axes, variable_out_axes),
-      axis_name=axis_name,
-      axis_size=axis_size,
-      spmd_axis_name=spmd_axis_name,
+        jax.vmap,
+        in_axes=(variable_in_axes, rng_axes, in_axes),
+        out_axes=(out_axes, variable_out_axes),
+        axis_name=axis_name,
+        axis_size=axis_size,
+        spmd_axis_name=spmd_axis_name,
     )
     @functools.wraps(fn)
     def mapped(variable_groups, rng_groups, args):
@@ -841,6 +848,7 @@ def vmap(
       y = fn(scope, *args)
       return y, repack_fn(scope)
 
+    # optional user-defined variable transform on the way out
     y, vars_out = mapped(variable_groups, rng_groups, args)
     new_vars_out = []
     for var_group, axis in zip(vars_out, variable_out_axes):
@@ -867,6 +875,7 @@ def scan(
   length: Optional[int] = None,
   reverse: bool = False,
   unroll: int = 1,
+  _split_transpose: bool = False,
   data_transform: Optional[Callable[..., Any]] = None,
   metadata_params: Dict[Any, Any] = {},
 ) -> Callable[..., Any]:
@@ -930,6 +939,8 @@ def scan(
     reverse: If true, scan from end to start in reverse order.
     unroll: how many scan iterations to unroll within a single
       iteration of a loop (default: 1).
+    _split_transpose: An experimental feature to split the transpose of a scan
+       into a scan and a map, backed by an experimental Jax lax.scan() feature.
     data_transform: optional function to transform raw variable and rng groups,
       intended for inline SPMD annotations.
     metadata_params: arguments dict passed to AxisMetadata instances in the
@@ -969,27 +980,33 @@ def scan(
       raise ValueError('length should be specified manually.')
     else:
       d_length = length
-    split_fn = lambda rng: random.split(rng, d_length)
+    # random.clone is only available on Jax versions 0.4.26 or newer
+    # see: https://jax.readthedocs.io/en/latest/jax.experimental.key_reuse.html
+    if hasattr(random, 'clone'):
+      split_fn = lambda rng: random.split(random.clone(rng), d_length)
+    else:
+      split_fn = lambda rng: random.split(rng, d_length)
 
     rng_groups = tuple(
-      tree_map_rngs(split_fn, rng_group) if split else rng_group
-      for rng_group, split in zip(rng_groups, rng_splits)
+        tree_map_rngs(split_fn, rng_group) if split else rng_group
+        for rng_group, split in zip(rng_groups, rng_splits)
     )
 
     @functools.partial(
-      axes_scan.scan,
-      in_axes=(variable_in_axes, rng_axes, in_axes),
-      out_axes=(out_axes, variable_out_axes),
-      length=length,
-      reverse=reverse,
-      unroll=unroll,
+        axes_scan.scan,
+        in_axes=(variable_in_axes, rng_axes, in_axes),
+        out_axes=(out_axes, variable_out_axes),
+        length=length,
+        reverse=reverse,
+        unroll=unroll,
+        _split_transpose=_split_transpose
     )
     def scanned(broadcast_vars, carry, scan_variable_groups, rng_groups, args):
       carry_vars, c = carry
       variable_groups = (broadcast_vars, carry_vars) + scan_variable_groups
       if data_transform is not None:
         variable_groups, rng_groups = data_transform(
-          variable_groups, rng_groups
+            variable_groups, rng_groups
         )
       scope = scope_fn(variable_groups, rng_groups)
       c, y = fn(scope, c, *args)
