@@ -1,4 +1,4 @@
-# Copyright 2023 The Flax Authors.
+# Copyright 2024 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -50,15 +50,18 @@ class Template(FlaxError):
 
 
 class FlaxError(Exception):
-
   def __init__(self, message):
     error_page = (
-        'https://flax.readthedocs.io/en/latest/api_reference/flax.errors.html'
+      'https://flax.readthedocs.io/en/latest/api_reference/flax.errors.html'
     )
     module_name = self.__class__.__module__
     class_name = self.__class__.__name__
-    error_msg = f'{message} ({error_page}#{module_name}.{class_name})'
-    super().__init__(error_msg)
+    if error_page not in message: # do not add a FlaxError link on unpickling
+      message = f'{message} ({error_page}#{module_name}.{class_name})'
+    super().__init__(message)
+
+  def __reduce__(self):
+   return (FlaxError, (str(self),))
 
 
 #################################################
@@ -88,8 +91,8 @@ class LazyInitError(FlaxError):
 
   def __init__(self, partial_val):
     super().__init__(
-        'Lazy init encountered a value that could with '
-        f'the given inputs (shape: {partial_val}).'
+      'Lazy init encountered a value that could with '
+      f'the given inputs (shape: {partial_val}).'
     )
 
 
@@ -168,14 +171,14 @@ class ApplyScopeInvalidVariablesTypeError(FlaxError):
 
   def __init__(self):
     super().__init__(
-        'The first argument passed to an apply function should be '
-        'a dictionary of collections. Each collection should be a '
-        'dictionary with string keys.'
+      'The first argument passed to an apply function should be '
+      'a dictionary of collections. Each collection should be a '
+      'dictionary with string keys.'
     )
 
 
 class ApplyScopeInvalidVariablesStructureError(FlaxError):
-  """This error is thrown when the dict passed as `variables` to apply() has an
+  """This error is thrown when the dict passed as ``variables`` to apply() has an
 
   extra 'params' layer, i.e. {'params': {'params': ...}}.
   For more explanation on variable dicts, please see :mod:`flax.core.variables`.
@@ -183,10 +186,10 @@ class ApplyScopeInvalidVariablesStructureError(FlaxError):
 
   def __init__(self, variables):
     super().__init__(
-        f'Expect the `variables` (first argument) passed to apply() '
-        f'to be a dict with the structure {{"params": ...}}, but got a dict '
-        f'with an extra params layer, i.e.  {{"params": {{"params": ... }} }}. '
-        f'You should instead pass in your dict\'s ["params"].'
+      f'Expect the `variables` (first argument) passed to apply() '
+      f'to be a dict with the structure {{"params": ...}}, but got a dict '
+      f'with an extra params layer, i.e.  {{"params": {{"params": ... }} }}. '
+      f'You should instead pass in your dict\'s ["params"].'
     )
 
 
@@ -215,8 +218,8 @@ class ScopeParamNotFoundError(FlaxError):
 
   def __init__(self, param_name, scope_path):
     super().__init__(
-        f'Could not find parameter named "{param_name}" in scope '
-        f'"{scope_path}".'
+      f'Could not find parameter named "{param_name}" in scope '
+      f'"{scope_path}".'
     )
 
 
@@ -225,6 +228,7 @@ class ScopeCollectionNotFound(FlaxError):
   collection.
 
   There are two common causes:
+
   1. | The collection was not passed to ``apply`` correctly.
      | For example, you might have used ``module.apply(params, ...)`` instead
      | of ``module.apply({'params': params}, ...)``.
@@ -235,8 +239,8 @@ class ScopeCollectionNotFound(FlaxError):
 
   def __init__(self, col_name, var_name, scope_path):
     super().__init__(
-        f'Tried to access "{var_name}" from collection "{col_name}" in '
-        f'"{scope_path}" but the collection is empty.'
+      f'Tried to access "{var_name}" from collection "{col_name}" in '
+      f'"{scope_path}" but the collection is empty.'
     )
 
 
@@ -270,9 +274,9 @@ class ScopeParamShapeError(FlaxError):
 
   def __init__(self, param_name, scope_path, value_shape, init_shape):
     super().__init__(
-        f'Initializer expected to generate shape {init_shape} '
-        f'but got shape {value_shape} instead for parameter '
-        f'"{param_name}" in "{scope_path}".'
+      f'Initializer expected to generate shape {init_shape} '
+      f'but got shape {value_shape} instead for parameter '
+      f'"{param_name}" in "{scope_path}".'
     )
 
 
@@ -286,8 +290,8 @@ class ScopeVariableNotFoundError(FlaxError):
 
   def __init__(self, name, col, scope_path):
     super().__init__(
-        f'No Variable named "{name}" for collection "{col}" '
-        f'exists in "{scope_path}".'
+      f'No Variable named "{name}" for collection "{col}" '
+      f'exists in "{scope_path}".'
     )
 
 
@@ -333,8 +337,8 @@ class ModifyScopeVariableError(FlaxError):
 
   def __init__(self, col, variable_name, scope_path):
     super().__init__(
-        f'Cannot update variable "{variable_name}" in '
-        f'"{scope_path}" because collection "{col}" is immutable.'
+      f'Cannot update variable "{variable_name}" in '
+      f'"{scope_path}" because collection "{col}" is immutable.'
     )
 
 
@@ -365,8 +369,8 @@ class PartitioningUnspecifiedError(FlaxError):
 
   def __init__(self, target):
     super().__init__(
-        'Trying to transform a Partitioned variable but "partition_name"'
-        f' is not specified in metadata_params: {target}'
+      'Trying to transform a Partitioned variable but "partition_name"'
+      f' is not specified in metadata_params: {target}'
     )
 
 
@@ -427,8 +431,8 @@ class NameInUseError(FlaxError):
   def __init__(self, key_type, value, module_name):
     # key_type is in {param, variable, submodule}.
     super().__init__(
-        f'Could not create {key_type} "{value}" in Module '
-        f'{module_name}: Name in use.'
+      f'Could not create {key_type} "{value}" in Module '
+      f'{module_name}: Name in use.'
     )
 
 
@@ -472,8 +476,8 @@ class AssignSubModuleError(FlaxError):
 
   def __init__(self, cls):
     super().__init__(
-        f'Submodule {cls} must be defined in `setup()` or in a '
-        'method wrapped in `@compact`'
+      f'Submodule {cls} must be defined in `setup()` or in a '
+      'method wrapped in `@compact`'
     )
 
 
@@ -546,9 +550,9 @@ class SetAttributeFrozenModuleError(FlaxError):
 
   def __init__(self, module_cls, attr_name, attr_val):
     super().__init__(
-        f"Can't set {attr_name}={attr_val} for Module of type "
-        f'{module_cls}: Module instance is frozen outside of '
-        'setup method.'
+      f"Can't set {attr_name}={attr_val} for Module of type "
+      f'{module_cls}: Module instance is frozen outside of '
+      'setup method.'
     )
 
 
@@ -562,7 +566,7 @@ class MultipleMethodsCompactError(FlaxError):
   * Use two separate modules that both have a unique ``@compact`` method.
 
   TODO(marcvanzee): Link to a design note explaining the motivation behind this.
-  There is no need for an equivalent to `hk.transparent` and it makes submodules
+  There is no need for an equivalent to ``hk.transparent`` and it makes submodules
   much more sane because there is no need to prefix the method names.
   """
 
@@ -581,7 +585,7 @@ class ReservedModuleAttributeError(FlaxError):
 
   def __init__(self, annotations):
     super().__init__(
-        f'properties `parent` and `name` are reserved: {annotations}'
+      f'properties `parent` and `name` are reserved: {annotations}'
     )
 
 
@@ -598,7 +602,7 @@ class ApplyModuleInvalidMethodError(FlaxError):
 
   def __init__(self, method):
     super().__init__(
-        f'Cannot call apply(): {method} is not a valid function for apply().'
+      f'Cannot call apply(): {method} is not a valid function for apply().'
     )
 
 
@@ -629,7 +633,7 @@ class CallCompactUnboundModuleError(FlaxError):
 
 
 class CallSetupUnboundModuleError(FlaxError):
-  """This error occurs when you are trying to call `.setup()` directly.
+  """This error occurs when you are trying to call ``.setup()`` directly.
 
   For instance, the error will be raised when trying to run this code::
 
@@ -644,12 +648,12 @@ class CallSetupUnboundModuleError(FlaxError):
     module.setup() # <-- ERROR!
     submodule = module.submodule
 
-  In general you shouldn't call `.setup()` yourself, if you need to get access
-  to a field or submodule defined inside `setup` you can instead create a
+  In general you shouldn't call ``.setup()`` yourself, if you need to get access
+  to a field or submodule defined inside ``setup`` you can instead create a
   function
-  to extract it and pass it to `nn.apply`::
+  to extract it and pass it to ``nn.apply``::
 
-    # setup() will be called automatically by `nn.apply`
+    # setup() will be called automatically by ``nn.apply``
     def get_submodule(module):
       return module.submodule.clone() # avoid leaking the Scope
 
@@ -689,7 +693,7 @@ class CallUnbindOnUnboundModuleError(FlaxError):
 
 
 class InvalidInstanceModuleError(FlaxError):
-  """This error occurs when you are trying to call `.init()`, `.init_with_output()`, `.apply() or `.bind()`
+  """This error occurs when you are trying to call ``.init()``, ``.init_with_output()``, ``.apply()`` or ``.bind()``
 
   on the Module class itself, instead of an instance of the Module class.
   For example, the error will be raised when trying to run this code::
@@ -708,13 +712,13 @@ class InvalidInstanceModuleError(FlaxError):
 
   def __init__(self):
     super().__init__(
-        'Can only call init, init_with_output or apply methods on an instance'
-        ' of the Module class, not the Module class itself'
+      'Can only call init, init_with_output or apply methods on an instance'
+      ' of the Module class, not the Module class itself'
     )
 
 
 class IncorrectPostInitOverrideError(FlaxError):
-  """This error occurs when you overrode `.__post_init__()` without calling `super().__post_init__()`.
+  """This error occurs when you overrode ``.__post_init__()`` without calling ``super().__post_init__()``.
 
   For example, the error will be raised when trying to run this code::
 
@@ -736,7 +740,7 @@ class IncorrectPostInitOverrideError(FlaxError):
 
   def __init__(self):
     super().__init__(
-        'Overrode `.__post_init__()` without calling `super().__post_init__()`'
+      'Overrode `.__post_init__()` without calling `super().__post_init__()`'
     )
 
 
@@ -758,8 +762,8 @@ class DescriptorAttributeError(FlaxError):
 
   def __init__(self):
     super().__init__(
-        'Trying to access a property that is accessing a non-existent'
-        ' attribute.'
+      'Trying to access a property that is accessing a non-existent'
+      ' attribute.'
     )
 
 
@@ -774,8 +778,8 @@ class InvalidCheckpointError(FlaxError):
 
   def __init__(self, path, step):
     super().__init__(
-        f'Trying to save an outdated checkpoint at step: "{step}" and path:'
-        f' "{path}".'
+      f'Trying to save an outdated checkpoint at step: "{step}" and path:'
+      f' "{path}".'
     )
 
 
@@ -792,9 +796,9 @@ class MPACheckpointingRequiredError(FlaxError):
 
   def __init__(self, path, step):
     super().__init__(
-        f'Checkpoint failed at step: "{step}" and path: "{path}": Target '
-        'contains a multiprocess array should be saved/restored with a '
-        'GlobalAsyncCheckpointManager.'
+      f'Checkpoint failed at step: "{step}" and path: "{path}": Target '
+      'contains a multiprocess array should be saved/restored with a '
+      'GlobalAsyncCheckpointManager.'
     )
 
 
@@ -810,10 +814,10 @@ class MPARestoreTargetRequiredError(FlaxError):
 
   def __init__(self, path, step, key=None):
     error_msg = (
-        f'Restore checkpoint failed at step: "{step}" and path: "{path}": '
-        'Checkpoints containing a multiprocess array need to be restored with '
-        'a target with pre-created arrays. If you cannot provide a full valid '
-        'target, consider ``allow_partial_mpa_restoration=True``. '
+      f'Restore checkpoint failed at step: "{step}" and path: "{path}": '
+      'Checkpoints containing a multiprocess array need to be restored with '
+      'a target with pre-created arrays. If you cannot provide a full valid '
+      'target, consider ``allow_partial_mpa_restoration=True``. '
     )
     if key:
       error_msg += f'This error fired when trying to restore array at {key}.'
@@ -828,9 +832,9 @@ class MPARestoreDataCorruptedError(FlaxError):
 
   def __init__(self, step, path):
     super().__init__(
-        f'Restore checkpoint failed at step: "{step}" on multiprocess array at '
-        f' "{path}": No "commit_success.txt" found on this "_gda" directory. '
-        'Was its save halted before completion?'
+      f'Restore checkpoint failed at step: "{step}" on multiprocess array at '
+      f' "{path}": No "commit_success.txt" found on this "_gda" directory. '
+      'Was its save halted before completion?'
     )
 
 
@@ -844,7 +848,7 @@ class TransformedMethodReturnValueError(FlaxError):
 
   def __init__(self, name):
     super().__init__(
-        f'Transformed module method {name} cannot return Modules or Variables.'
+      f'Transformed module method {name} cannot return Modules or Variables.'
     )
 
 
@@ -857,7 +861,7 @@ class TransformTargetError(FlaxError):
 
     nn.vmap(nn.Dense(features))(x)  # raises TransformTargetError
 
-  You can transform the `nn.Dense` class directly instead::
+  You can transform the ``nn.Dense`` class directly instead::
 
     nn.vmap(nn.Dense)(features)(x)
 
@@ -875,9 +879,9 @@ class TransformTargetError(FlaxError):
 
   def __init__(self, target):
     super().__init__(
-        'Linen transformations must be applied to Modules classes or'
-        ' functions taking a Module instance as the first argument.'
-        f' The provided target is not a Module class or callable: {target}'
+      'Linen transformations must be applied to Modules classes or'
+      ' functions taking a Module instance as the first argument.'
+      f' The provided target is not a Module class or callable: {target}'
     )
 
 
@@ -912,9 +916,9 @@ class CursorFindError(FlaxError):
   def __init__(self, cursor=None, cursor2=None):
     if cursor and cursor2:
       super().__init__(
-          'More than one object found given the conditions of the cond_fn. '
-          'The first two objects found have the following paths: '
-          f'{cursor._path} and {cursor2._path}'
+        'More than one object found given the conditions of the cond_fn. '
+        'The first two objects found have the following paths: '
+        f'{cursor._path} and {cursor2._path}'
       )
     else:
       super().__init__('No object found given the conditions of the cond_fn.')
@@ -937,11 +941,11 @@ class TraverseTreeError(FlaxError):
   def __init__(self, update_fn, cond_fn):
     if update_fn is None and cond_fn is None:
       super().__init__(
-          'Both update_fn and cond_fn are None. Exactly one of them must be'
-          ' None.'
+        'Both update_fn and cond_fn are None. Exactly one of them must be'
+        ' None.'
       )
     else:
       super().__init__(
-          'Both update_fn and cond_fn are not None. Exactly one of them must be'
-          ' not None.'
+        'Both update_fn and cond_fn are not None. Exactly one of them must be'
+        ' not None.'
       )
