@@ -13,8 +13,10 @@
 # limitations under the License.
 
 """Attention core modules for Flax."""
+from __future__ import annotations
 
 import functools
+import inspect
 import warnings
 from typing import Any, Callable, Optional, Union, overload
 
@@ -42,17 +44,19 @@ from flax.typing import (
 
 
 def dot_product_attention_weights(
-  query: Array,
-  key: Array,
-  bias: Optional[Array] = None,
-  mask: Optional[Array] = None,
-  broadcast_dropout: bool = True,
-  dropout_rng: Optional[PRNGKey] = None,
-  dropout_rate: float = 0.0,
-  deterministic: bool = False,
-  dtype: Optional[Dtype] = None,
-  precision: PrecisionLike = None,
-  module: Optional[Module] = None,
+    query: Array,
+    key: Array,
+    bias: Optional[Array] = None,
+    mask: Optional[Array] = None,
+    broadcast_dropout: bool = True,
+    dropout_rng: Optional[PRNGKey] = None,
+    dropout_rate: float = 0.0,
+    deterministic: bool = False,
+    dtype: Optional[Dtype] = None,
+    precision: PrecisionLike = None,
+    module: Optional[Module] = None,
+    force_fp32_for_softmax: bool = False,
+    einsum_dot_general: Callable[..., Array] = jax.lax.dot_general,
 ):
   """Computes dot-product attention weights given query and key.
 
@@ -61,8 +65,8 @@ def dot_product_attention_weights(
   you can directly call this function and call einsum yourself.
 
   Args:
-    query: queries for calculating attention with shape of ``[batch..., q_length,
-      num_heads, qk_depth_per_head]``.
+    query: queries for calculating attention with shape of ``[batch...,
+      q_length, num_heads, qk_depth_per_head]``.
     key: keys for calculating attention with shape of ``[batch..., kv_length,
       num_heads, qk_depth_per_head]``.
     bias: bias for the attention weights. This should be broadcastable to the
@@ -80,9 +84,13 @@ def dot_product_attention_weights(
     precision: numerical precision of the computation see ``jax.lax.Precision``
       for details.
     module: the Module that will sow the attention weights into the
-      'intermediates' collection. Remember to mark 'intermediates' as mutable via
-      ``mutable=['intermediates']`` in order to have that collection returned.
-      If ``module`` is None, the attention weights will not be sowed.
+      'intermediates' collection. Remember to mark 'intermediates' as mutable
+      via ``mutable=['intermediates']`` in order to have that collection
+      returned. If ``module`` is None, the attention weights will not be sowed.
+    force_fp32_for_softmax: bool, whether to force the softmax to be computed in
+      fp32. This is useful for mixed-precision training where higher precision
+      is desired for numerical stability.
+    einsum_dot_general: the dot_general to use in einsum.
 
   Returns:
     Output of shape ``[batch..., num_heads, q_length, kv_length]``.
@@ -100,7 +108,11 @@ def dot_product_attention_weights(
   query = query / jnp.sqrt(depth).astype(dtype)
   # attn weight shape is (batch..., num_heads, q_length, kv_length)
   attn_weights = jnp.einsum(
-    '...qhd,...khd->...hqk', query, key, precision=precision
+      '...qhd,...khd->...hqk',
+      query,
+      key,
+      precision=precision,
+      _dot_general=einsum_dot_general,
   )
 
   # apply attention bias: masking, dropout, proximity bias, etc.
@@ -112,7 +124,10 @@ def dot_product_attention_weights(
     attn_weights = jnp.where(mask, attn_weights, big_neg)
 
   # normalize the attention weights
-  attn_weights = jax.nn.softmax(attn_weights).astype(dtype)
+  if force_fp32_for_softmax and dtype != jnp.float32:
+    attn_weights = jax.nn.softmax(attn_weights.astype(jnp.float32))
+  else:
+    attn_weights = jax.nn.softmax(attn_weights).astype(dtype)
 
   if module:
     module.sow('intermediates', 'attention_weights', attn_weights)
@@ -133,18 +148,20 @@ def dot_product_attention_weights(
 
 
 def dot_product_attention(
-  query: Array,
-  key: Array,
-  value: Array,
-  bias: Optional[Array] = None,
-  mask: Optional[Array] = None,
-  broadcast_dropout: bool = True,
-  dropout_rng: Optional[PRNGKey] = None,
-  dropout_rate: float = 0.0,
-  deterministic: bool = False,
-  dtype: Optional[Dtype] = None,
-  precision: PrecisionLike = None,
-  module: Optional[Module] = None,
+    query: Array,
+    key: Array,
+    value: Array,
+    bias: Optional[Array] = None,
+    mask: Optional[Array] = None,
+    broadcast_dropout: bool = True,
+    dropout_rng: Optional[PRNGKey] = None,
+    dropout_rate: float = 0.0,
+    deterministic: bool = False,
+    dtype: Optional[Dtype] = None,
+    precision: PrecisionLike = None,
+    module: Optional[Module] = None,
+    force_fp32_for_softmax: bool = False,
+    einsum_dot_general: Callable[..., Array] = jax.lax.dot_general,
 ):
   """Computes dot-product attention given query, key, and value.
 
@@ -152,11 +169,12 @@ def dot_product_attention(
   https://arxiv.org/abs/1706.03762. It calculates the attention weights given
   query and key and combines the values using the attention weights.
 
-  Note: query, key, value needn't have any batch dimensions.
+  .. note::
+    ``query``, ``key``, ``value`` needn't have any batch dimensions.
 
   Args:
-    query: queries for calculating attention with shape of ``[batch..., q_length,
-      num_heads, qk_depth_per_head]``.
+    query: queries for calculating attention with shape of ``[batch...,
+      q_length, num_heads, qk_depth_per_head]``.
     key: keys for calculating attention with shape of ``[batch..., kv_length,
       num_heads, qk_depth_per_head]``.
     value: values to be used in attention with shape of ``[batch..., kv_length,
@@ -176,9 +194,13 @@ def dot_product_attention(
     precision: numerical precision of the computation see ``jax.lax.Precision`
       for details.
     module: the Module that will sow the attention weights into the
-      'intermediates' collection. Remember to mark 'intermediates' as mutable via
-      ``mutable=['intermediates']`` in order to have that collection returned.
-      If ``module`` is None, the attention weights will not be sowed.
+      'intermediates' collection. Remember to mark 'intermediates' as mutable
+      via ``mutable=['intermediates']`` in order to have that collection
+      returned. If ``module`` is None, the attention weights will not be sowed.
+    force_fp32_for_softmax: bool, whether to force the softmax to be computed in
+      fp32. This is useful for mixed-precision training where higher precision
+      is desired for numerical stability.
+    einsum_dot_general: the dot_general to use in einsum.
 
   Returns:
     Output of shape ``[batch..., q_length, num_heads, v_depth_per_head]``.
@@ -196,22 +218,28 @@ def dot_product_attention(
 
   # compute attention weights
   attn_weights = dot_product_attention_weights(
-    query,
-    key,
-    bias,
-    mask,
-    broadcast_dropout,
-    dropout_rng,
-    dropout_rate,
-    deterministic,
-    dtype,
-    precision,
-    module,
+      query,
+      key,
+      bias,
+      mask,
+      broadcast_dropout,
+      dropout_rng,
+      dropout_rate,
+      deterministic,
+      dtype,
+      precision,
+      module,
+      force_fp32_for_softmax,
+      einsum_dot_general=einsum_dot_general,
   )
 
   # return weighted sum over values for each query position
   return jnp.einsum(
-    '...hqk,...khd->...qhd', attn_weights, value, precision=precision
+      '...hqk,...khd->...qhd',
+      attn_weights,
+      value,
+      precision=precision,
+      _dot_general=einsum_dot_general,
   )
 
 
@@ -267,26 +295,30 @@ class MultiHeadDotProductAttention(Module):
     >>> out3, out4 = module.apply(variables, q, rngs={'dropout': key6}, dropout_rng=key5)
 
   Attributes:
-    num_heads: number of attention heads. Features (i.e. inputs_q.shape[-1])
+    num_heads: Number of attention heads. Features (i.e. inputs_q.shape[-1])
       should be divisible by the number of heads.
-    dtype: the dtype of the computation (default: infer from inputs and params)
-    param_dtype: the dtype passed to parameter initializers (default: float32)
-    qkv_features: dimension of the key, query, and value.
-    out_features: dimension of the last projection
-    broadcast_dropout: bool: use a broadcasted dropout along batch dims.
-    dropout_rate: dropout rate
-    deterministic: if false, the attention weight is masked randomly using
-      dropout, whereas if true, the attention weights are deterministic.
-    precision: numerical precision of the computation see ``jax.lax.Precision``
+    dtype: The dtype of the computation (default: infer from inputs and params)
+    param_dtype: The dtype passed to parameter initializers (default: float32)
+    qkv_features: Dimension of the key, query, and value.
+    out_features: Dimension of the last projection
+    broadcast_dropout: Use a broadcasted dropout along batch dims.
+    dropout_rate: Dropout rate.
+    deterministic: If False, the attention weight is masked randomly using
+      dropout, whereas if True, the attention weights are deterministic.
+    precision: Numerical precision of the computation see ``jax.lax.Precision``
       for details.
-    kernel_init: initializer for the kernel of the Dense layers.
-    bias_init: initializer for the bias of the Dense layers.
-    use_bias: bool: whether pointwise QKVO dense transforms use bias.
+    kernel_init: Initializer for the kernel of the Dense layers.
+    out_kernel_init: Optional Initializer for the kernel of the output Dense layer,
+      if None, ``kernel_init`` will be used.
+    bias_init: Initializer for the bias of the Dense layers.
+    out_bias_init: Optional Initializer for the bias of the output Dense layer,
+      if None, ``bias_init`` will be used.
+    use_bias: Whether pointwise QKVO dense transforms use bias.
     attention_fn: dot_product_attention or compatible function. Accepts query,
       key, value, and returns output of shape ``[bs, dim1, dim2, ..., dimN,,
       num_heads, value_channels]``
-    decode: whether to prepare and use an autoregressive cache.
-    normalize_qk: should QK normalization be applied (arxiv.org/abs/2302.05442).
+    decode: Whether to prepare and use an autoregressive cache.
+    normalize_qk: Should QK normalization be applied (arxiv.org/abs/2302.05442).
   """
 
   num_heads: int
@@ -299,11 +331,14 @@ class MultiHeadDotProductAttention(Module):
   deterministic: Optional[bool] = None
   precision: PrecisionLike = None
   kernel_init: Initializer = default_kernel_init
+  out_kernel_init: Initializer | None = None
   bias_init: Initializer = initializers.zeros_init()
+  out_bias_init: Initializer | None = None
   use_bias: bool = True
   attention_fn: Callable[..., Array] = dot_product_attention
   decode: bool = False
   normalize_qk: bool = False
+  force_fp32_for_softmax: bool = False
   # Deprecated, will be removed.
   qkv_dot_general: Optional[DotGeneralT] = None
   out_dot_general: Optional[DotGeneralT] = None
@@ -459,8 +494,18 @@ class MultiHeadDotProductAttention(Module):
     if self.normalize_qk:
       # Normalizing query and key projections stabilizes training with higher
       # LR. See ViT-22B paper http://arxiv.org/abs/2302.05442 for analysis.
-      query = LayerNorm(name='query_ln', use_bias=False)(query)  # type: ignore[call-arg]
-      key = LayerNorm(name='key_ln', use_bias=False)(key)  # type: ignore[call-arg]
+      query = LayerNorm(
+        name='query_ln',
+        use_bias=False,
+        dtype=self.dtype,
+        param_dtype=self.param_dtype,
+      )(query)  # type: ignore[call-arg]
+      key = LayerNorm(
+        name='key_ln',
+        use_bias=False,
+        dtype=self.dtype,
+        param_dtype=self.param_dtype,
+      )(key)  # type: ignore[call-arg]
 
     # During fast autoregressive decoding, we feed one position at a time,
     # and cache the keys and values step by step.
@@ -494,7 +539,9 @@ class MultiHeadDotProductAttention(Module):
         # update key, value caches with our new 1d spatial slices
         cur_index = cache_index.value
         zero = jnp.array(0, dtype=lax.dtype(cur_index.dtype))
-        indices: tuple[Union[int, jax.Array], ...] = (zero,) * len(batch_dims) + (
+        indices: tuple[Union[int, jax.Array], ...] = (zero,) * len(
+          batch_dims
+        ) + (
           cur_index,
           zero,
           zero,
@@ -528,39 +575,34 @@ class MultiHeadDotProductAttention(Module):
       m_deterministic = True
 
     # apply attention
+    attn_args = (query, key, value)
+    # This kwargs list match the default nn.dot_product_attention.
+    # For custom `attention_fn`s, invalid kwargs will be filtered.
+    attn_kwargs = dict(
+      mask=mask,
+      dropout_rng=dropout_rng,
+      dropout_rate=self.dropout_rate,
+      broadcast_dropout=self.broadcast_dropout,
+      deterministic=m_deterministic,
+      dtype=self.dtype,
+      precision=self.precision,
+      force_fp32_for_softmax=self.force_fp32_for_softmax,
+    )
+    attn_kwargs = {
+        k: v
+        for k, v in attn_kwargs.items()
+        if k in inspect.signature(self.attention_fn).parameters
+    }
     if sow_weights:
-      x = self.attention_fn(
-        query,
-        key,
-        value,
-        mask=mask,
-        dropout_rng=dropout_rng,
-        dropout_rate=self.dropout_rate,
-        broadcast_dropout=self.broadcast_dropout,
-        deterministic=m_deterministic,
-        dtype=self.dtype,
-        precision=self.precision,
-        module=self,
-      )  # pytype: disable=wrong-keyword-args
+      x = self.attention_fn(*attn_args, **attn_kwargs, module=self)
     else:
-      x = self.attention_fn(
-        query,
-        key,
-        value,
-        mask=mask,
-        dropout_rng=dropout_rng,
-        dropout_rate=self.dropout_rate,
-        broadcast_dropout=self.broadcast_dropout,
-        deterministic=m_deterministic,
-        dtype=self.dtype,
-        precision=self.precision,
-      )
+      x = self.attention_fn(*attn_args, **attn_kwargs)
     # back to the original inputs dimensions
     out = DenseGeneral(
       features=features,
       axis=(-2, -1),
-      kernel_init=self.kernel_init,
-      bias_init=self.bias_init,
+      kernel_init=self.out_kernel_init or self.kernel_init,
+      bias_init=self.out_bias_init or self.bias_init,
       use_bias=self.use_bias,
       dtype=self.dtype,
       param_dtype=self.param_dtype,
@@ -699,7 +741,11 @@ class SelfAttention(MultiHeadDotProductAttention):
       DeprecationWarning,
     )
     return super().__call__(
-      inputs_q, mask=mask, deterministic=deterministic, dropout_rng=dropout_rng, sow_weights=sow_weights
+      inputs_q,
+      mask=mask,
+      deterministic=deterministic,
+      dropout_rng=dropout_rng,
+      sow_weights=sow_weights,
     )
 
 
@@ -766,7 +812,9 @@ def make_causal_mask(
   )
 
 
-def combine_masks(*masks: Optional[Array], dtype: Dtype = jnp.float32) -> Optional[Array]:
+def combine_masks(
+  *masks: Optional[Array], dtype: Dtype = jnp.float32
+) -> Optional[Array]:
   """Combine attention masks.
 
   Args:
