@@ -14,7 +14,6 @@
 
 """Linear modules."""
 
-import dataclasses
 from typing import (
   Any,
   Iterable,
@@ -31,9 +30,12 @@ import numpy as np
 from jax import eval_shape, lax
 from jax.core import ShapedArray
 
+import opt_einsum
+
 from flax.core import meta
 from flax.linen import initializers
 from flax.linen.dtypes import promote_dtype
+from flax.linen import module
 from flax.linen.module import Module, compact
 from flax.typing import (
   Array,
@@ -77,12 +79,12 @@ class DenseGeneral(Module):
     >>> # output features (4, 5)
     >>> layer = nn.DenseGeneral(features=(4, 5))
     >>> params = layer.init(jax.random.key(0), jnp.ones((1, 3)))
-    >>> jax.tree_map(jnp.shape, params)
+    >>> jax.tree_util.tree_map(jnp.shape, params)
     {'params': {'bias': (4, 5), 'kernel': (3, 4, 5)}}
     >>> # apply transformation on the the second and last axes
     >>> layer = nn.DenseGeneral(features=(4, 5), axis=(1, -1))
     >>> params = layer.init(jax.random.key(0), jnp.ones((1, 3, 6, 7)))
-    >>> jax.tree_map(jnp.shape, params)
+    >>> jax.tree_util.tree_map(jnp.shape, params)
     {'params': {'bias': (4, 5), 'kernel': (3, 7, 4, 5)}}
 
   Attributes:
@@ -145,7 +147,7 @@ class DenseGeneral(Module):
         * np.prod(shape[n_batch_dims : n_axis + n_batch_dims]),
         np.prod(shape[-n_features:]),
       )
-      flat_shape = jax.tree_map(int, flat_shape)
+      flat_shape = jax.tree_util.tree_map(int, flat_shape)
       kernel = self.kernel_init(rng, flat_shape, dtype)
       if isinstance(kernel, meta.AxisMetadata):
         return meta.replace_boxed(kernel, jnp.reshape(kernel.unbox(), shape))
@@ -172,7 +174,7 @@ class DenseGeneral(Module):
         flat_shape = (
           np.prod(shape[:n_batch_dims]) * np.prod(shape[-n_features:]),
         )
-        flat_shape = jax.tree_map(int, flat_shape)
+        flat_shape = jax.tree_util.tree_map(int, flat_shape)
         bias = self.bias_init(rng, flat_shape, dtype)
         if isinstance(bias, meta.AxisMetadata):
           return meta.replace_boxed(bias, jnp.reshape(bias.unbox(), shape))
@@ -216,7 +218,7 @@ class Dense(Module):
 
     >>> layer = nn.Dense(features=4)
     >>> params = layer.init(jax.random.key(0), jnp.ones((1, 3)))
-    >>> jax.tree_map(jnp.shape, params)
+    >>> jax.tree_util.tree_map(jnp.shape, params)
     {'params': {'bias': (4,), 'kernel': (3, 4)}}
 
   Attributes:
@@ -282,6 +284,128 @@ class Dense(Module):
     return y
 
 
+class Einsum(Module):
+  """An einsum transformation with learnable kernel and bias.
+
+  Example usage::
+
+    >>> import flax.linen as nn
+    >>> import jax, jax.numpy as jnp
+
+    >>> layer = nn.Einsum((5, 6, 7), 'abc,cde->abde')
+    >>> variables = layer.init(jax.random.key(0), jnp.ones((3, 4, 5)))
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
+    {'params': {'bias': (6, 7), 'kernel': (5, 6, 7)}}
+
+  Attributes:
+    shape: the shape of the kernel.
+    einsum_str: a string to denote the einsum equation. The equation must
+      have exactly two operands, the lhs being the input passed in, and
+      the rhs being the learnable kernel. Exactly one of ``einsum_str``
+      in the constructor argument and call argument must be not None,
+      while the other must be None.
+    use_bias: whether to add a bias to the output (default: True).
+    dtype: the dtype of the computation (default: infer from input and params).
+    param_dtype: the dtype passed to parameter initializers (default: float32).
+    precision: numerical precision of the computation see ``jax.lax.Precision``
+      for details.
+    kernel_init: initializer function for the weight matrix.
+    bias_init: initializer function for the bias.
+  """
+
+  shape: Shape
+  einsum_str: Optional[str] = None
+  use_bias: bool = True
+  dtype: Optional[Dtype] = None
+  param_dtype: Dtype = jnp.float32
+  precision: PrecisionLike = None
+  kernel_init: Initializer = default_kernel_init
+  bias_init: Initializer = initializers.zeros_init()
+
+  @compact
+  def __call__(self, inputs: Array, einsum_str: Optional[str] = None) -> Array:
+    """Applies a linear transformation to the inputs along the last dimension.
+
+    Args:
+      inputs: The nd-array to be transformed.
+      einsum_str: a string to denote the einsum equation. The equation must
+        have exactly two operands, the lhs being the input passed in, and
+        the rhs being the learnable kernel. The ``einsum_str`` passed into
+        the call method will take precedence over the ``einsum_str`` passed
+        into the constructor.
+
+    Returns:
+      The transformed input.
+    """
+    einsum_str = module.merge_param('einsum_str', self.einsum_str, einsum_str)
+
+    einsum_str = einsum_str.replace(' ', '')
+    if '->' not in einsum_str:
+      raise ValueError(
+        '`einsum_str` equation must be explicit and include "->".'
+      )
+    if einsum_str.count(',') != 1:
+      raise ValueError(
+        '`einsum_str` equation must have exactly two operands and '
+        'therefore, exactly one comma character, instead of '
+        f'{einsum_str.count(",")}'
+      )
+
+    kernel = self.param(
+      'kernel',
+      self.kernel_init,
+      self.shape,
+      self.param_dtype,
+    )
+
+    if self.use_bias:
+      bias_shape, broadcasted_bias_shape = self._get_bias_shape(
+        einsum_str, inputs, kernel
+      )
+      bias = self.param('bias', self.bias_init, bias_shape, self.param_dtype)
+    else:
+      bias = None
+
+    inputs, kernel, bias = promote_dtype(inputs, kernel, bias, dtype=self.dtype)
+
+    y = jnp.einsum(einsum_str, inputs, kernel, precision=self.precision)
+
+    if bias is not None:
+      y += jnp.reshape(bias, broadcasted_bias_shape)
+    return y
+
+  def _get_bias_shape(self, einsum_str: str, lhs: Array, rhs: Array):
+    """Infer the bias shape and broadcasted bias shape given the ``einsum_str``,
+    ``lhs`` and ``rhs`` arrays. This is needed for instantiating the bias
+    parameter and adding the bias to the output during forward inference.
+
+    This function first replaces all ellipses with actual letter characters,
+    then computes the bias shape by checking to see which axes in the rhs
+    array remain in the resulting array after einsumming. These axes are the
+    embedding/feature dimensions, and all other axes in rhs are reduction axes.
+    """
+    # More details on the parsing function: https://github.com/dgasmith/opt_einsum/blob/c826bb7df16f470a69f7bf90598fc27586209d11/opt_einsum/parser.py#L246
+    # returns the einsum string representation of the operands and result, with
+    # ellipsis replaced by actual letter characters
+    operands_str, result_str, _ = opt_einsum.parser.parse_einsum_input(
+      (einsum_str, lhs, rhs)
+    )
+
+    # rhs_dict is a dict{character:index} mapping that maps every character in
+    # the rhs einsum string representation to its corresponding index position in the string
+    rhs_dict = {c: i for i, c in enumerate(operands_str.split(',')[1])}
+    assert len(rhs_dict) == len(self.shape)
+
+    broadcasted_bias_shape = [1] * len(result_str)
+    bias_shape = []
+    for i, c in enumerate(result_str):
+      if c in rhs_dict:
+        broadcasted_bias_shape[i] = self.shape[rhs_dict[c]]
+        bias_shape.append(self.shape[rhs_dict[c]])
+
+    return bias_shape, broadcasted_bias_shape
+
+
 def _conv_dimension_numbers(input_shape):
   """Computes the dimension numbers based on the input shape."""
   ndim = len(input_shape)
@@ -291,7 +415,6 @@ def _conv_dimension_numbers(input_shape):
   return lax.ConvDimensionNumbers(lhs_spec, rhs_spec, out_spec)
 
 
-
 def canonicalize_padding(padding: PaddingLike, rank: int) -> LaxPadding:
   """ "Canonicalizes conv padding to a jax.lax supported format."""
   if isinstance(padding, str):
@@ -317,7 +440,7 @@ def canonicalize_padding(padding: PaddingLike, rank: int) -> LaxPadding:
 
 
 class _Conv(Module):
-  """Convolution Module wrapping ``lax.conv_general_dilated[_local]``.
+  """Convolution Module wrapping ``lax.conv_general_dilated``.
 
   Attributes:
     features: number of convolution filters.
@@ -388,11 +511,10 @@ class _Conv(Module):
     """Applies a (potentially unshared) convolution to the inputs.
 
     Args:
-      inputs: input data with dimensions (*batch_dims, spatial_dims...,
-        features). This is the channels-last convention, i.e. NHWC for a 2d
-        convolution and NDHWC for a 3D convolution. Note: this is different from
-        the input convention used by ``lax.conv_general_dilated``, which puts the
-        spatial dimensions last.
+      inputs: input data with dimensions ``(*batch_dims, spatial_dims..., features)``.
+        This is the channels-last convention, i.e. NHWC for a 2d convolution and
+        NDHWC for a 3D convolution. Note: this is different from the input convention
+        used by ``lax.conv_general_dilated``, which puts the spatial dimensions last.
         Note: If the input has more than 1 batch dimension, all batch dimensions
         are flattened into a single dimension for the convolution and restored
         before returning.  In some cases directly vmap'ing the layer may yield
@@ -583,14 +705,14 @@ class Conv(_Conv):
     >>> # valid padding
     >>> layer = nn.Conv(features=4, kernel_size=(3,), padding='VALID')
     >>> out, variables = layer.init_with_output(jax.random.key(0), jnp.ones((1, 8, 3)))
-    >>> jax.tree_map(jnp.shape, variables)
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
     {'params': {'bias': (4,), 'kernel': (3, 3, 4)}}
     >>> out.shape
     (1, 6, 4)
     >>> # circular padding with stride 2
     >>> layer = nn.Conv(features=4, kernel_size=(3, 3), strides=2, padding='CIRCULAR')
     >>> out, variables = layer.init_with_output(jax.random.key(0), jnp.ones((1, 8, 3)))
-    >>> jax.tree_map(jnp.shape, variables)
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
     {'params': {'bias': (4,), 'kernel': (3, 3, 3, 4)}}
     >>> out.shape
     (1, 4, 4)
@@ -606,25 +728,26 @@ class Conv(_Conv):
     strides: an integer or a sequence of `n` integers, representing the
       inter-window strides (default: 1).
     padding: either the string ``'SAME'``, the string ``'VALID'``, the string
-      ``'CIRCULAR'`` (periodic boundary conditions), or a sequence of ``n`` ``(low,
-      high)`` integer pairs that give the padding to apply before and after each
-      spatial dimension. A single int is interpreted as applying the same padding
-      in all dims and assign a single int in a sequence causes the same padding
-      to be used on both sides. ``'CAUSAL'`` padding for a 1D convolution will
-      left-pad the convolution axis, resulting in same-sized output.
+      ``'CIRCULAR'`` (periodic boundary conditions), or a sequence of ``n``
+      ``(low, high)`` integer pairs that give the padding to apply before and
+      after each spatial dimension. A single int is interpreted as applying the
+      same padding in all dims and assign a single int in a sequence causes the
+      same padding to be used on both sides. ``'CAUSAL'`` padding for a 1D
+      convolution will left-pad the convolution axis, resulting in same-sized
+      output.
     input_dilation: an integer or a sequence of ``n`` integers, giving the
-      dilation factor to apply in each spatial dimension of ``inputs``
-      (default: 1). Convolution with input dilation ``d`` is equivalent to
-      transposed convolution with stride ``d``.
+      dilation factor to apply in each spatial dimension of ``inputs`` (default:
+      1). Convolution with input dilation ``d`` is equivalent to transposed
+      convolution with stride ``d``.
     kernel_dilation: an integer or a sequence of ``n`` integers, giving the
       dilation factor to apply in each spatial dimension of the convolution
-      kernel (default: 1). Convolution with kernel dilation
-      is also known as 'atrous convolution'.
+      kernel (default: 1). Convolution with kernel dilation is also known as
+      'atrous convolution'.
     feature_group_count: integer, default 1. If specified divides the input
       features into groups.
     use_bias: whether to add a bias to the output (default: True).
     mask: Optional mask for the weights during masked convolution. The mask must
-          be the same shape as the convolution weight matrix.
+      be the same shape as the convolution weight matrix.
     dtype: the dtype of the computation (default: infer from input and params).
     param_dtype: the dtype passed to parameter initializers (default: float32).
     precision: numerical precision of the computation see ``jax.lax.Precision`
@@ -649,14 +772,14 @@ class ConvLocal(_Conv):
     >>> # valid padding
     >>> layer = nn.ConvLocal(features=4, kernel_size=(3,), padding='VALID')
     >>> out, variables = layer.init_with_output(jax.random.key(0), jnp.ones((1, 8, 3)))
-    >>> jax.tree_map(jnp.shape, variables)
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
     {'params': {'bias': (6, 4), 'kernel': (6, 9, 4)}}
     >>> out.shape
     (1, 6, 4)
     >>> # circular padding with stride 2
     >>> layer = nn.ConvLocal(features=4, kernel_size=(3, 3), strides=2, padding='CIRCULAR')
     >>> out, variables = layer.init_with_output(jax.random.key(0), jnp.ones((1, 8, 3)))
-    >>> jax.tree_map(jnp.shape, variables)
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
     {'params': {'bias': (1, 4, 4), 'kernel': (1, 4, 27, 4)}}
     >>> out.shape
     (1, 4, 4)
@@ -672,25 +795,26 @@ class ConvLocal(_Conv):
     strides: an integer or a sequence of `n` integers, representing the
       inter-window strides (default: 1).
     padding: either the string ``'SAME'``, the string ``'VALID'``, the string
-      ``'CIRCULAR'`` (periodic boundary conditions), or a sequence of ``n`` ``(low,
-      high)`` integer pairs that give the padding to apply before and after each
-      spatial dimension. A single int is interpreted as applying the same padding
-      in all dims and assign a single int in a sequence causes the same padding
-      to be used on both sides. ``'CAUSAL'`` padding for a 1D convolution will
-      left-pad the convolution axis, resulting in same-sized output.
+      ``'CIRCULAR'`` (periodic boundary conditions), or a sequence of ``n``
+      ``(low, high)`` integer pairs that give the padding to apply before and
+      after each spatial dimension. A single int is interpreted as applying the
+      same padding in all dims and assign a single int in a sequence causes the
+      same padding to be used on both sides. ``'CAUSAL'`` padding for a 1D
+      convolution will left-pad the convolution axis, resulting in same-sized
+      output.
     input_dilation: an integer or a sequence of ``n`` integers, giving the
-      dilation factor to apply in each spatial dimension of ``inputs``
-      (default: 1). Convolution with input dilation ``d`` is equivalent to
-      transposed convolution with stride ``d``.
+      dilation factor to apply in each spatial dimension of ``inputs`` (default:
+      1). Convolution with input dilation ``d`` is equivalent to transposed
+      convolution with stride ``d``.
     kernel_dilation: an integer or a sequence of ``n`` integers, giving the
       dilation factor to apply in each spatial dimension of the convolution
-      kernel (default: 1). Convolution with kernel dilation
-      is also known as 'atrous convolution'.
+      kernel (default: 1). Convolution with kernel dilation is also known as
+      'atrous convolution'.
     feature_group_count: integer, default 1. If specified divides the input
       features into groups.
     use_bias: whether to add a bias to the output (default: True).
     mask: Optional mask for the weights during masked convolution. The mask must
-          be the same shape as the convolution weight matrix.
+      be the same shape as the convolution weight matrix.
     dtype: the dtype of the computation (default: infer from input and params).
     param_dtype: the dtype passed to parameter initializers (default: float32).
     precision: numerical precision of the computation see ``jax.lax.Precision``
@@ -705,7 +829,7 @@ class ConvLocal(_Conv):
 
 
 class ConvTranspose(Module):
-  """Convolution Module wrapping lax.conv_transpose.
+  """Convolution Module wrapping ``lax.conv_transpose``.
 
   Example usage::
 
@@ -715,14 +839,14 @@ class ConvTranspose(Module):
     >>> # valid padding
     >>> layer = nn.ConvTranspose(features=4, kernel_size=(3,), padding='VALID')
     >>> out, variables = layer.init_with_output(jax.random.key(0), jnp.ones((1, 8, 3)))
-    >>> jax.tree_map(jnp.shape, variables)
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
     {'params': {'bias': (4,), 'kernel': (3, 3, 4)}}
     >>> out.shape
     (1, 10, 4)
     >>> # circular padding with stride 2
     >>> layer = nn.ConvTranspose(features=4, kernel_size=(6, 6), strides=(2, 2), padding='CIRCULAR', transpose_kernel=True)
     >>> out, variables = layer.init_with_output(jax.random.key(0), jnp.ones((1, 15, 15, 3)))
-    >>> jax.tree_map(jnp.shape, variables)
+    >>> jax.tree_util.tree_map(jnp.shape, variables)
     {'params': {'bias': (4,), 'kernel': (6, 6, 4, 3)}}
     >>> out.shape
     (1, 30, 30, 4)
@@ -733,31 +857,32 @@ class ConvTranspose(Module):
 
   Attributes:
     features: number of convolution filters.
-    kernel_size: shape of the convolutional kernel. For 1D convolution,
-      the kernel size can be passed as an integer, which will be interpreted
-      as a tuple of the single integer. For all other cases, it must be a
-      sequence of integers.
-    strides: a sequence of `n` integers, representing the inter-window strides.
+    kernel_size: shape of the convolutional kernel. For 1D convolution, the
+      kernel size can be passed as an integer, which will be interpreted as a
+      tuple of the single integer. For all other cases, it must be a sequence of
+      integers.
+    strides: an integer or a sequence of `n` integers, representing the
+      inter-window strides.
     padding: either the string `'SAME'`, the string `'VALID'`, the string
       `'CIRCULAR'` (periodic boundary conditions), or a sequence of `n` `(low,
       high)` integer pairs that give the padding to apply before and after each
-      spatial dimension. A single int is interpreted as applying the same padding
-      in all dims and assign a single int in a sequence causes the same padding
-      to be used on both sides.
-    kernel_dilation: ``None``, or a sequence of ``n`` integers, giving the
-      dilation factor to apply in each spatial dimension of the convolution
+      spatial dimension. A single int is interpreted as applying the same
+      padding in all dims and assign a single int in a sequence causes the same
+      padding to be used on both sides.
+    kernel_dilation: ``None``, or an integer or a sequence of ``n`` integers,
+      giving the dilation factor to apply in each spatial dimension of the convolution
       kernel. Convolution with kernel dilation is also known as 'atrous
       convolution'.
     use_bias: whether to add a bias to the output (default: True).
     mask: Optional mask for the weights during masked convolution. The mask must
-          be the same shape as the convolution weight matrix.
+      be the same shape as the convolution weight matrix.
     dtype: the dtype of the computation (default: infer from input and params).
     param_dtype: the dtype passed to parameter initializers (default: float32).
     precision: numerical precision of the computation see ``jax.lax.Precision``
       for details.
     kernel_init: initializer for the convolutional kernel.
     bias_init: initializer for the bias.
-    transpose_kernel: if True flips spatial axes and swaps the input/output
+    transpose_kernel: if ``True`` flips spatial axes and swaps the input/output
       channel axes of the kernel.
   """
 
@@ -782,11 +907,10 @@ class ConvTranspose(Module):
     Behaviour mirrors of ``jax.lax.conv_transpose``.
 
     Args:
-      inputs: input data with dimensions (*batch_dims, spatial_dims...,
-        features). This is the channels-last convention, i.e. NHWC for a 2d
-        convolution and NDHWC for a 3D convolution. Note: this is different from
-        the input convention used by ``lax.conv_general_dilated``, which puts the
-        spatial dimensions last.
+      inputs: input data with dimensions ``(*batch_dims, spatial_dims..., features).``
+        This is the channels-last convention, i.e. NHWC for a 2d convolution and NDHWC
+        for a 3D convolution. Note: this is different from the input convention used by
+        ``lax.conv_general_dilated``, which puts the spatial dimensions last.
         Note: If the input has more than 1 batch dimension, all batch dimensions
         are flattened into a single dimension for the convolution and restored
         before returning.  In some cases directly vmap'ing the layer may yield
@@ -803,6 +927,17 @@ class ConvTranspose(Module):
     else:
       kernel_size = tuple(self.kernel_size)
 
+    def maybe_broadcast(
+      x: Optional[Union[int, Sequence[int]]],
+    ) -> Tuple[int, ...]:
+      if x is None:
+        # backward compatibility with using None as sentinel for
+        # broadcast 1
+        x = 1
+      if isinstance(x, int):
+        return (x,) * len(kernel_size)
+      return tuple(x)
+
     # Combine all input batch dimensions into a single leading batch axis.
     num_batch_dimensions = inputs.ndim - (len(kernel_size) + 1)
     if num_batch_dimensions != 1:
@@ -813,11 +948,8 @@ class ConvTranspose(Module):
       ]
       inputs = jnp.reshape(inputs, flat_input_shape)
 
-    strides: Tuple[int, ...]
-    if self.strides is None:
-      strides = (1,) * (inputs.ndim - 2)
-    else:
-      strides = tuple(self.strides)
+    strides = maybe_broadcast(self.strides)
+    kernel_dilation = maybe_broadcast(self.kernel_dilation)
 
     in_features = jnp.shape(inputs)[-1]
     if self.transpose_kernel:
@@ -856,7 +988,7 @@ class ConvTranspose(Module):
       kernel,
       strides,
       padding_lax,
-      rhs_dilation=self.kernel_dilation,
+      rhs_dilation=kernel_dilation,
       transpose_kernel=self.transpose_kernel,
       precision=self.precision,
     )
@@ -970,8 +1102,6 @@ class Embed(Module):
   param_dtype: Dtype = jnp.float32
   embedding_init: Initializer = default_embed_init
 
-  embedding: Array = dataclasses.field(init=False)
-
   def setup(self):
     self.embedding = self.param(
       'embedding',
