@@ -34,7 +34,7 @@ from typing import Any, Callable, List, Optional, Sequence, Tuple, Union
 
 import jax
 from jax import lax
-from jax.experimental import maps
+from jax.interpreters import pxla
 
 from flax import struct
 from flax.core import meta
@@ -107,9 +107,9 @@ def _mesh_assignment_free(new_assignment, existing_assignments):
 
 
 def _logical_to_mesh_axes(
-  array_dim_names: Optional[Sequence[Optional[str]]],
-  rules: Optional[LogicalRules] = None,
-) -> Optional[List[Union[_UnassignedAxis, None, str, Tuple[str]]]]:
+    array_dim_names: Optional[Sequence[Optional[str]]],
+    rules: Optional[LogicalRules] = None,
+) -> Optional[List[Union[_UnassignedAxis, None, str, Tuple[str, ...]]]]:
   """Same as logical_to_mesh_axes, but doesn't fill in _unassigned_axis."""
   if array_dim_names is None:
     return None
@@ -126,8 +126,11 @@ def _logical_to_mesh_axes(
   if not isinstance(rules, (tuple, list)):
     raise ValueError('Unknown axis rule specification type.')
   # We assign mesh axes using a priority based ruleset over logical axis names.
-  result: List[Union[_UnassignedAxis, None, str, Tuple[str]]]
-  result = [_unassigned_axis] * len(array_dim_names)
+  result: List[Union[_UnassignedAxis, None, str, Tuple[str, ...]]]
+  result = [
+      (_unassigned_axis if isinstance(name, str) else name)
+      for name in array_dim_names
+  ]
   for rule_model_name, rule_mesh_names in rules:
     if rule_model_name in array_dim_names:
       pos = array_dim_names.index(rule_model_name)
@@ -188,10 +191,10 @@ def logical_to_mesh_axes(
 
 def logical_to_mesh(tree: Any, rules: Optional[LogicalRules] = None) -> Any:
   """Applies logical_to_mesh_axes to pytrees of logical PartitionSpecs."""
-  return jax.tree_map(
-    lambda x: logical_to_mesh_axes(x, rules),
-    tree,
-    is_leaf=lambda x: isinstance(x, jax.sharding.PartitionSpec),
+  return jax.tree_util.tree_map(
+      lambda x: logical_to_mesh_axes(x, rules),
+      tree,
+      is_leaf=lambda x: isinstance(x, jax.sharding.PartitionSpec),
   )
 
 
@@ -201,17 +204,17 @@ def logical_to_mesh_sharding(
   rules: Optional[LogicalRules] = None,
 ) -> Any:
   """Convert pytrees of logical PartitionSpecs to shardings."""
-  return jax.tree_map(
-    lambda x: jax.sharding.NamedSharding(mesh, x),
-    logical_to_mesh(tree, rules),
-    is_leaf=lambda x: isinstance(x, jax.sharding.PartitionSpec),
+  return jax.tree_util.tree_map(
+      lambda x: jax.sharding.NamedSharding(mesh, x),
+      logical_to_mesh(tree, rules),
+      is_leaf=lambda x: isinstance(x, jax.sharding.PartitionSpec),
   )
 
 
 def _global_mesh_defined() -> bool:
-  """Checks if global xmap/jit mesh resource environment is defined."""
-  maps_env = maps.thread_resources.env
-  return maps_env.physical_mesh.devices.shape != ()  # pylint: disable=g-explicit-bool-comparison
+  """Checks if global mesh resource environment is defined."""
+  env = pxla.thread_resources.env
+  return env.physical_mesh.devices.shape != ()  # pylint: disable=g-explicit-bool-comparison
 
 
 class RulesFallback(enum.Enum):
@@ -264,9 +267,17 @@ def _with_sharding_constraint_one_fallback(
   )
 
 
+def _is_axis_spec(x):
+  return (
+      isinstance(x, str)
+      or x is jax.sharding.PartitionSpec.UNCONSTRAINED
+      or x is None
+  )
+
+
 def _is_logical_spec(x):
   return x is None or (
-    isinstance(x, tuple) and all(isinstance(e, str) or e is None for e in x)
+      isinstance(x, tuple) and all(_is_axis_spec(e) for e in x)
   )
 
 
