@@ -336,6 +336,8 @@ def tabulate(
 def _get_flops(fn, *args, **kwargs):
   e = jax.jit(fn).lower(*args, **kwargs)
   cost = e.cost_analysis()
+  if cost is None:
+    return 0
   flops = int(cost['flops']) if 'flops' in cost else 0
   return flops
 
@@ -363,12 +365,12 @@ def _get_call_flops(
   if not compute_flops and not compute_vjp_flops:
     return -1, -1
 
-  rngs = jax.tree_map(
-    lambda x: x.rng, c.rngs, is_leaf=lambda x: isinstance(x, LazyRng)
+  rngs = jax.tree_util.tree_map(
+      lambda x: x.rng, c.rngs, is_leaf=lambda x: isinstance(x, LazyRng)
   )
 
-  args = jax.tree_map(_from_value_representation, c.args)
-  kwargs = jax.tree_map(_from_value_representation, c.kwargs)
+  args = jax.tree_util.tree_map(_from_value_representation, c.args)
+  kwargs = jax.tree_util.tree_map(_from_value_representation, c.kwargs)
 
   leaves, treedef = jax.tree_util.tree_flatten((args, kwargs))
   dynamic_leaves = []
@@ -458,7 +460,7 @@ def _get_module_table(
       call_depth = len(c.path)
       inputs = _process_inputs(c.args, c.kwargs)
 
-      if c.path in visited_paths:
+      if c.path in visited_paths or not hasattr(c.module, c.method):
         if not show_repeated:
           continue
         module_vars = {}
