@@ -36,6 +36,7 @@ versions as "lifted transformations".
 A lifted transformation can be applied to a ``Module`` class or a
 function that takes a ``Module`` instance as its first argument.
 """
+
 import dataclasses
 import functools
 import inspect
@@ -53,6 +54,7 @@ from typing import (
   Union,
 )
 
+from flax import core
 from flax import errors, struct, traceback_util
 from flax import serialization
 from flax.core import Scope, lift, meta
@@ -460,86 +462,111 @@ class _HashableProxy:
   """
 
   module: Module
+  hash_key: int
+
+  @classmethod
+  def from_module(cls, module: Module) -> '_HashableProxy':
+    fingerprint = _module_fingerprint(module)
+    hash_key = hash(fingerprint)
+    return cls(module, hash_key)
 
   def __hash__(self):
-    key = _module_pytree_hash(self.module)
-    return hash(key)
+    return self.hash_key
 
   def __eq__(self, other):
-    return isinstance(other, _HashableProxy) and _module_pytree_hash(
-        self.module
-    ) == _module_pytree_hash(other.module)
+    return isinstance(other, _HashableProxy) and self.hash_key == other.hash_key
 
 
-def _recursive_freeze(name: str, x: Any, seen_modules: dict[FlaxId, int]):
-  if isinstance(x, dict):
-    return tuple(
-        (k, _recursive_freeze(k, v, seen_modules)) for k, v in x.items()
-    )
-  elif isinstance(x, Module):
-    return _module_pytree_hash_recursive(x, seen_modules)
+def _module_fingerprint(module: Module) -> tuple[type[Any], Any]:
+  return _fingerprint_recursive(module, (), {})
+
+
+def _fingerprint_recursive(
+  obj: Any, path: tuple[str, ...], seen_modules: dict[FlaxId, int]
+) -> Any:
+  """Creates a hashable representation for a Module by traversing its structure recursively."""
+
+  def _get_fingerprint(name: str, value: Any) -> tuple[str, Any]:
+    return name, _fingerprint_recursive(value, (*path, name), seen_modules)
+
+  if isinstance(obj, str):
+    return obj
+  elif isinstance(obj, Module):
+    fingerprint: Any
+    if obj._id in seen_modules:
+      # if we have already seen the module we just use the index
+      # as its static component
+      fingerprint = seen_modules[obj._id]
+      return type(obj), fingerprint
+    else:
+      # if its a new module we add it to the cache and give it
+      # a new index
+      seen_modules[obj._id] = len(seen_modules)
+      # TODO(cgarciae): define a way for the user of nn.jit to define
+      # what fields it wants to ignore per Module instance.
+      fingerprints = []
+      for field in dataclasses.fields(obj):
+        if not hasattr(obj, field.name):
+          continue
+        if field.name not in ('parent', 'name'):
+          value = getattr(obj, field.name)
+          fingerprints.append(_get_fingerprint(field.name, value))
+      # add state fingerprint
+      state_fingerprint = (
+        _get_fingerprint('in_compact_method', obj._state.in_compact_method),
+        _get_fingerprint('in_setup', obj._state.in_setup),
+        _get_fingerprint('setup_called', obj._state.setup_called),
+        _get_fingerprint('is_initialized', obj._state.is_initialized),
+        _get_fingerprint('autoname_cursor', obj._state.autoname_cursor),
+      )
+      fingerprints.append(('_state', state_fingerprint))
+      # add scope fingerprint
+      scope = obj.scope
+      if scope is not None:
+        static_scope = (
+          _get_fingerprint('mutable', scope.mutable),
+          _get_fingerprint('flags', scope.flags),
+          _get_fingerprint('rng_counts', scope.rng_counters),
+          _get_fingerprint('reservations', scope.reservations),
+        )
+        _check_field_is_hashable((*path, 'scope'), static_scope)
+        fingerprints.append(('scope', static_scope))
+      fingerprint = tuple(fingerprints)
+      return type(obj), fingerprint
+  elif dataclasses.is_dataclass(obj):
+    fingerprints = []
+    for field in dataclasses.fields(obj):
+      if not hasattr(obj, field.name):
+        continue
+      value = getattr(obj, field.name)
+      value_fingerprint = _get_fingerprint(field.name, value)
+      fingerprints.append((field.name, value_fingerprint))
+    return type(obj), tuple(fingerprints)
+  elif isinstance(obj, core.DenyList):
+    return type(obj), _get_fingerprint('deny', obj.deny)
+  elif isinstance(obj, dict):
+    fingerprint = tuple((k, _get_fingerprint(k, v)) for k, v in obj.items())
+    return fingerprint
+  elif serialization.is_serializable(obj):
+    state = serialization.to_state_dict(obj)
+    fingerprint = _fingerprint_recursive(state, path, seen_modules)
+    return type(obj), fingerprint
+  elif isinstance(obj, Mapping):
+    return tuple((k, _get_fingerprint(k, v)) for k, v in obj.items())
+  elif isinstance(obj, Iterable):
+    return tuple(_get_fingerprint(str(i), v) for i, v in enumerate(obj))
   else:
-    _check_field_is_hashable(name, x)
-    return x
+    _check_field_is_hashable(path, obj)
+    return obj
 
 
-def _module_pytree_hash(module: Module) -> tuple[type[Any], Any]:
-  return _module_pytree_hash_recursive(module, {})
-
-
-def _module_pytree_hash_recursive(
-    module: Module, seen_modules: dict[FlaxId, int]
-) -> tuple[type[Any], Any]:
-  """Creates a hashable representation for a Module by travering its structure recursively."""
-  static: Any
-  if module._id in seen_modules:
-    # if we have already seen the module we just use the index
-    # as its static component
-    static = seen_modules[module._id]
-  else:
-    # if its a new module we add it to the cache and give it
-    # a new index
-    seen_modules[module._id] = len(seen_modules)
-    # TODO(cgarciae): define a way for the user of nn.jit to define
-    # what fields it wants to ignore per Module instance.
-    values = []
-    for field in dataclasses.fields(module):
-      if field.init and field.name not in ('parent', 'name'):
-        value = getattr(module, field.name)
-        if isinstance(value, Module):
-          value = _module_pytree_hash_recursive(value, seen_modules)
-        elif serialization.is_serializable(value):
-          value = serialization.to_state_dict(value)
-          value = _recursive_freeze(field.name, value, seen_modules)
-        else:
-          _check_field_is_hashable(field.name, value)
-        values.append((field.name, value))
-    static = tuple(values)
-
-  return type(module), static
-
-
-def _check_field_is_hashable(name: str, x: Any):
+def _check_field_is_hashable(path: tuple[str, ...], x: Any):
   """Checks if a field is hashable."""
   try:
     hash(x)
   except Exception as e:
-    if dataclasses.is_dataclass(x):
-      if x.__hash__ is None:
-        raise ValueError(
-            f"field '{name}' of type '{type(x)}' is a dataclass but its not"
-            ' hashable, using `dataclass(frozen=True, eq=True)` to make it'
-            ' immutable and hashable, or `dataclass(unsafe_hash=True)` for'
-            ' mutable types.'
-        ) from e
-      else:
-        raise ValueError(
-            f"field '{name}' of type '{type(x)}' is a hashable dataclass but"
-            ' hashing failed, this probably means that at least one of its'
-            ' fields is not hashable.'
-        ) from e
-    else:
-      raise ValueError(f"type '{type(x)}' is not hashable.") from e
+    path_name = '/'.join(path)
+    raise ValueError(f"Value at '{path_name}' is not hashable: {e}") from e
 
 
 def decorator_lift_transform_jit(class_fn, **trafo_kwargs):
@@ -569,12 +596,12 @@ def decorator_lift_transform_jit(class_fn, **trafo_kwargs):
 
     # make a scope-function to transform
     def core_fn(
-        prewrapped_fn,
-        class_fn,
-        scopes,
-        module_hash,
-        *args,
-        **kwargs,
+      prewrapped_fn,
+      class_fn,
+      scopes,
+      module_hash,
+      *args,
+      **kwargs,
     ):
       # self = hash_key.obj
       self: Module = module_hash.module
@@ -588,8 +615,8 @@ def decorator_lift_transform_jit(class_fn, **trafo_kwargs):
       return res
 
     core_fns = [
-        functools.partial(core_fn, prewrapped_fn, class_fn)
-        for prewrapped_fn, class_fn in zip(prewrapped_fns, class_fns)
+      functools.partial(core_fn, prewrapped_fn, class_fn)
+      for prewrapped_fn, class_fn in zip(prewrapped_fns, class_fns)
     ]
 
     # here we apply the given lifting transform to the scope-ingesting fn
@@ -605,13 +632,13 @@ def decorator_lift_transform_jit(class_fn, **trafo_kwargs):
         # constructors. Therefore, there is no obvious API for specifying
         # arguments per lifted Module.
         raise NotImplementedError(
-            'This transform does not yet support'
-            ' Modules that include other Modules passed as arguments.'
+          'This transform does not yet support'
+          ' Modules that include other Modules passed as arguments.'
         )
       module_scopes = module_scopes[0]
 
-    # get a hash for the Module by using its repr as a proxy
-    hash_key = _HashableProxy(self)
+    # get a hashable proxy object for the Module
+    hash_key = _HashableProxy.from_module(self)
 
     return trafo_fn(module_scopes, hash_key, *args, **kwargs)
 
@@ -637,12 +664,12 @@ def module_class_lift_transform_jit(module_class, methods=None, **trafo_kwargs):
     class_trafo_args = {k: ((), v) for k, v in methods.items()}
   else:
     raise ValueError(
-        'transform methods argument must be None, tuple, list, or dict.'
+      'transform methods argument must be None, tuple, list, or dict.'
     )
 
   # Handle partially initialized module class constructors.
   if isinstance(module_class, functools.partial) and issubclass(
-      module_class.func, Module
+    module_class.func, Module
   ):
     partial_object = module_class
     module_class = module_class.func
@@ -666,9 +693,9 @@ def module_class_lift_transform_jit(module_class, methods=None, **trafo_kwargs):
         self: Module = module_hash.module
         # make a clone of self using its arguments
         attrs = {
-            f.name: getattr(self, f.name)
-            for f in dataclasses.fields(self)
-            if f.name != 'parent' and f.init
+          f.name: getattr(self, f.name)
+          for f in dataclasses.fields(self)
+          if f.name != 'parent' and f.init
         }
         # we reference module_class, not self.__class__ to avoid infinite loop
         cloned = module_class(parent=None, **attrs)
@@ -684,7 +711,7 @@ def module_class_lift_transform_jit(module_class, methods=None, **trafo_kwargs):
       module_scopes, args, kwargs = get_module_scopes(self, args, kwargs)
 
       # get a hash for the Module by using its repr as a proxy
-      hash_key = _HashableProxy(self)
+      hash_key = _HashableProxy.from_module(self)
 
       ret = trafo_fn(module_scopes, hash_key, *args, **kwargs)
       return ret
@@ -692,19 +719,19 @@ def module_class_lift_transform_jit(module_class, methods=None, **trafo_kwargs):
     return wrapped_fn
 
   transformed_fns = {
-      fn_name: create_trans_fn(fn_name, fn_trafo_args)
-      for fn_name, fn_trafo_args in class_trafo_args.items()
+    fn_name: create_trans_fn(fn_name, fn_trafo_args)
+    for fn_name, fn_trafo_args in class_trafo_args.items()
   }
   # construct new dynamic class w. transformed methods
   transformed_cls = type(
-      transform.__name__.capitalize() + module_class.__name__,
-      (module_class,),
-      transformed_fns,
+    transform.__name__.capitalize() + module_class.__name__,
+    (module_class,),
+    transformed_fns,
   )
   # Handle partially initialized module class constructors.
   if partial_object is not None:
     transformed_cls = functools.partial(
-        transformed_cls, *partial_object.args, **partial_object.keywords
+      transformed_cls, *partial_object.args, **partial_object.keywords
     )
   return transformed_cls
 
@@ -864,15 +891,15 @@ def vmap(
 
 
 def jit(
-    target: Target,
-    variables: CollectionFilter = True,
-    rngs: PRNGSequenceFilter = True,
-    static_argnums: Union[int, Iterable[int]] = (),
-    static_argnames: Union[str, Iterable[str]] = (),
-    donate_argnums: Union[int, Iterable[int]] = (),
-    device=None,
-    backend: Union[str, None] = None,
-    methods=None,
+  target: Target,
+  variables: CollectionFilter = True,
+  rngs: PRNGSequenceFilter = True,
+  static_argnums: Union[int, Iterable[int]] = (),
+  static_argnames: Union[str, Iterable[str]] = (),
+  donate_argnums: Union[int, Iterable[int]] = (),
+  device=None,
+  backend: Union[str, None] = None,
+  methods=None,
 ) -> Target:
   """Lifted version of ``jax.jit``.
 
@@ -921,27 +948,27 @@ def jit(
   # TODO(marcvanzee): Improve docstrings (#1977).
   if _is_module_class(target):
     return module_class_lift_transform_jit(
-        target,
-        variables=variables,
-        rngs=rngs,
-        static_argnums=static_argnums,
-        static_argnames=static_argnames,
-        donate_argnums=donate_argnums,
-        device=device,
-        backend=backend,
-        methods=methods,
+      target,
+      variables=variables,
+      rngs=rngs,
+      static_argnums=static_argnums,
+      static_argnames=static_argnames,
+      donate_argnums=donate_argnums,
+      device=device,
+      backend=backend,
+      methods=methods,
     )
   # we presume this is being used as a function decorator in class definition
   elif callable(target) and not isinstance(target, Module):
     return decorator_lift_transform_jit(
-        target,
-        variables=variables,
-        rngs=rngs,
-        static_argnums=static_argnums,
-        static_argnames=static_argnames,
-        donate_argnums=donate_argnums,
-        device=device,
-        backend=backend,
+      target,
+      variables=variables,
+      rngs=rngs,
+      static_argnums=static_argnums,
+      static_argnames=static_argnames,
+      donate_argnums=donate_argnums,
+      device=device,
+      backend=backend,
     )
   else:
     raise errors.TransformTargetError(target)
@@ -971,12 +998,12 @@ def checkpoint(
     >>> import flax.linen as nn
     ...
     >>> class CheckpointedMLP(nn.Module):
+    ...   @nn.checkpoint
     ...   @nn.compact
     ...   def __call__(self, x):
-    ...     CheckpointDense = nn.checkpoint(nn.Dense)
-    ...     x = CheckpointDense(128)(x)
+    ...     x = nn.Dense(128)(x)
     ...     x = nn.relu(x)
-    ...     x = CheckpointDense(1)(x)
+    ...     x = nn.Dense(1)(x)
     ...     return x
     ...
     >>> model = CheckpointedMLP()
@@ -1043,9 +1070,9 @@ def remat_scan(
   policy: Optional[Callable[..., bool]] = None,
   variable_broadcast: CollectionFilter = False,
   variable_carry: CollectionFilter = False,
-  variable_axes: Mapping[
-    CollectionFilter, InOutScanAxis
-  ] = FrozenDict({True: 0}),
+  variable_axes: Mapping[CollectionFilter, InOutScanAxis] = FrozenDict(
+    {True: 0}
+  ),
   split_rngs: Mapping[PRNGSequenceFilter, bool] = FrozenDict({True: True}),
 ) -> Target:
   """Combines remat and scan for memory efficiency and constant time compilation.
@@ -1105,9 +1132,7 @@ def remat_scan(
 
 def scan(
   target: Target,
-  variable_axes: Mapping[
-    CollectionFilter, InOutScanAxis
-  ] = FrozenDict(),
+  variable_axes: Mapping[CollectionFilter, InOutScanAxis] = FrozenDict(),
   variable_broadcast: CollectionFilter = False,
   variable_carry: CollectionFilter = False,
   split_rngs: Mapping[PRNGSequenceFilter, bool] = FrozenDict(),
@@ -1119,6 +1144,7 @@ def scan(
   data_transform: Optional[Callable[..., Any]] = None,
   metadata_params: Mapping[Any, Any] = {},
   methods=None,
+  _split_transpose: bool = False,
 ) -> Target:
   """A lifted version of ``jax.lax.scan``.
 
@@ -1256,6 +1282,8 @@ def scan(
     metadata_params: arguments dict passed to AxisMetadata instances in the
       variable tree.
     methods: If ``target`` is a ``Module``, the methods of ``Module`` to scan over.
+    _split_transpose: An experimental feature to split the transpose of a scan
+       into a scan and a map, backed by an experimental Jax lax.scan() feature.
 
   Returns:
     The scan function with the signature ``(module, carry, *xs) -> (carry,
@@ -1274,6 +1302,7 @@ def scan(
     length=length,
     reverse=reverse,
     unroll=unroll,
+    _split_transpose=_split_transpose,
     data_transform=data_transform,
     metadata_params=metadata_params,
     methods=methods,
@@ -1311,7 +1340,7 @@ def map_variables(
     ...   features: int
     ...
     ...   def apply_mask(self, variables):
-    ...     return (jax.tree_map(jnp.triu, variables)
+    ...     return (jax.tree_util.tree_map(jnp.triu, variables)
     ...             if not self.is_initializing() else variables)
     ...
     ...   def setup(self):
@@ -1337,8 +1366,9 @@ def map_variables(
     rngs: PRNGSequences added to the transformed scope (default: all).
     variables: Additional Variable collections added to the transformed scope.
       Besides those specified by ``target`` (default: all).
-    methods: If ``target`` is a ``Module``, the methods of ``Module`` to map variables
-      for.
+    methods: If ``target`` is a ``Module``, the methods of ``Module`` to map
+      variables for.
+
   Returns:
     a wrapped version of ``target`` that will map the specified collections.
   """
@@ -1758,8 +1788,8 @@ def while_loop(
 
     >>> k = jax.random.key(0)
     >>> x = jnp.ones((2, 2))
-    >>> intial_vars = WhileLoopExample().init(k, x)
-    >>> result, state = WhileLoopExample().apply(intial_vars, x, mutable=['state'])
+    >>> initial_vars = WhileLoopExample().init(k, x)
+    >>> result, state = WhileLoopExample().apply(initial_vars, x, mutable=['state'])
 
   Args:
     cond_fn: Should return True as long as the loop should continue.
